% ANISODIFF - Anisotropic diffusion.
%
% Usage:
%  diff = anisodiff(im, n_iterations, kappa, lambda, option)
%
%         im  - input image
%         n_iterations 
%         kappa - conduction coefficient 20-100 ?
%         lambda - max value of 0.25 for stability
%         option - 1 Perona Malik diffusion equation No 1
%                       2 Perona Malik diffusion equation No 2
%
% Reference: 
% P. Perona and J. Malik. 
% Scale-space and edge detection using ansotropic diffusion.
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 
% 12(7):629-639, July 1990.
%
% Author: Peter Kovesi   pk@cs.uwa.edu.au
% Department of Computer Science & Software Engineering
% The University of Western Australia
% June 2000         
%
%	EXAMPLE : anisodiff('zsmallOfficesmall.hdr',3,60,0.125,1);
%	[im,sens,ill,type]=radread('zsmallOfficesmall.hdr');
%	radwrite( im, 'test.hdr', type, '8-bit' , sens, ill);
%	pre- and post-process HDR with ra_rgbe.exe

function diff = anisodiff2(I, niterations, kappa, lambda, option)
%im = double(im);
	CIE_rf=.265074126; CIE_gr=.670114631;
	CIE_br=.064811243; MIN_LUM = 1.0e-3;
%	rgbeimage=radread(I);
	[rgbeimage,sens,ill,type]=radread(I);
	r=rgbeimage(:,:,1);
	g=rgbeimage(:,:,2);
	b=rgbeimage(:,:,3);
	%%%Get luminance value
	image = CIE_rf*r + CIE_gr*g + CIE_br*b;
%	The following does not work in a function, queing issues.
%	[s,smap]=cmunique(image);
%	v=sort(smap(:,1));
%	plot(v);
	image(find(image==0))=MIN_LUM;
	%%% transform luminance into logarithm domain.
	im = log(image);

[rows,cols] = size(im);
diff = im;
for i = 1:niterations
  fprintf('\rIteration %d',i);
  diffl = zeros(rows+2, cols+2);
  diffl(2:rows+1, 2:cols+1) = diff;
  deltaN = diffl(1:rows,2:cols+1) - diff;
  deltaS = diffl(3:rows+2,2:cols+1) - diff;
  deltaE = diffl(2:rows+1,3:cols+2) - diff;
  deltaW = diffl(2:rows+1,1:cols) - diff;
  if option == 1
    cN = exp(-(deltaN/kappa).^2);
    cS = exp(-(deltaS/kappa).^2);
    cE = exp(-(deltaE/kappa).^2);
    cW = exp(-(deltaW/kappa).^2);
  elseif option == 2
    cN = 1./(1+exp(-(deltaN/kappa).^2));
    cS = 1./(1+exp(-(deltaS/kappa).^2));
    cE = 1./(1+exp(-(deltaE/kappa).^2));
    cW = 1./(1+exp(-(deltaW/kappa).^2));
  end
  diff = diff + lambda*(cN.*deltaN + cS.*deltaS + cE.*deltaE + cW.*deltaW);
% END OF ORIGINAL PROGRAM
size(diff)
picture=diff; % ADDED BY ME - jlj
  imshow(picture);
%  Uncomment the following to see a progression of images
%  subplot(ceil(sqrt(niterations)),ceil(sqrt(niterations)), i)
%  imagesc(diff), colormap(gray), axis image

%tiffwrite(r,g,b,lhrfilename);
%radwrite( picture, 'test2.hdr', type, '8-bit' , sens, ill);

%[t,tmap]=cmunique(diff);
%u=sort(tmap(:,1));
%figure;
%plot(u);
%
end
fprintf('\n');

